package infopipes.pipes;

import infopipes.utils.Channel;
import infopipes.utils.Polarity;
import infopipes.utils.PolarityConfig;
import infopipes.utils.PolarityMap;
import infopipes.utils.Port;
import infopipes.utils.PortMap;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class CompositeLogicalPipe extends BaseLogicalPipe {
	Map<PolarityConfig, PolarityMap> internals;
	Map<String, LogicalPipe> pipes;
	Set<Channel> channels;
	
	public CompositeLogicalPipe(
			Map<String, Integer> inports,
			Map<String, Integer> outports,
			Map<PolarityConfig, PolarityMap> internals,
			Map<String, LogicalPipe> pipes,
			Set<Channel> channels) {
		this.inports = Collections.unmodifiableMap(inports);
		this.outports = Collections.unmodifiableMap(outports);
		this.internals = Collections.unmodifiableMap(internals);
		this.pipes = Collections.unmodifiableMap(pipes);
		this.channels = Collections.unmodifiableSet(channels);
		this.configs.addAll(internals.keySet());
	}

	@Override
	public CompositePhysicalPipe instantiate(PolarityConfig config) {
		PolarityMap internal;
		Map<String, PhysicalPipe> physical;
		PortMap inports, outports;
		
		if (!internals.containsKey(config))
			throw new RuntimeException();
		
		physical = new HashMap<String, PhysicalPipe>();
		internal = internals.get(config);
		for (String op : internal.keySet()) {
			PolarityConfig polarity = internal.get(op);
			physical.put(op, pipes.get(op).instantiate(polarity));
		}
		
		inports = new PortMap();
		outports = new PortMap();
		for (Channel c : channels) {
			final String srcName = c.src.pipe;
			final String destName = c.dest.pipe;
			final int srcIndex = pipes.get(srcName).outportIndex(c.src.ident);
			final int destIndex = pipes.get(destName).inportIndex(c.dest.ident);
			PhysicalPipe srcPipe = physical.get(srcName);
			PhysicalPipe destPipe = physical.get(destName);

			if (srcName.equals(CompositeFactory.ifaceName))
				inports.put(srcIndex, new Port(destPipe, destIndex));
			else if (destName.equals(CompositeFactory.ifaceName))
				outports.put(destIndex, new Port(srcPipe, srcIndex));
			else if (internal.get(srcName).content[srcIndex].equals(Polarity.P))
				srcPipe.linkPush(srcIndex, destPipe.getPush(destIndex));
			else
				destPipe.linkPull(srcIndex, srcPipe.getPull(srcIndex));
		}
		return new CompositePhysicalPipe(this, inports, outports, physical, config);
	}
}
