package infopipes.pipes;

import infopipes.utils.Polarity;
import infopipes.utils.PolarityConfig;
import infopipes.utils.Port;
import infopipes.utils.PortMap;

import java.util.Map;

public class CompositePhysicalPipe implements PhysicalPipe {
	final LogicalPipe logical;
	final Map<String, PhysicalPipe> pipes;
	final PortMap inports, outports;
	final Polarity[] config;
	
	public CompositePhysicalPipe(
			LogicalPipe logical,
			PortMap inports,
			PortMap outports,
			Map<String, PhysicalPipe> pipes,
			PolarityConfig config) {
		this.logical = logical;
		this.inports = inports;
		this.outports = outports;
		this.pipes = pipes;
		this.config = config.content;
	}
	
	public LogicalPipe getLogical() {
		return logical;
	}
	
	public PhysicalPipe getInternal(String ident) {
		return pipes.get(ident);
	}

	public Port getPull(int index) {
		return outports.get(index);
	}

	public Port getPush(int index) {
		return inports.get(index);
	}

	public void linkPull(int index, Port src) {
		Port dest = inports.get(index);
		dest.pipe.linkPull(dest.index, src);
	}

	public void linkPush(int index, Port dest) {
		Port src = outports.get(index);
		src.pipe.linkPush(src.index, dest);
	}
	
	public void connected() {
		for (String name : pipes.keySet())
			pipes.get(name).connected();
	}

	public Object pull(int index) {
		throw new RuntimeException();
	}

	public void push(int index, Object packet) {
		throw new RuntimeException();
	}
}
