package infopipes.pipes;

import infopipes.utils.PolarityChoice;
import infopipes.utils.PolarityConfig;
import infopipes.utils.PolaritySpace;

/**
 * This class, as its name suggests, is a dummy that cannot be
 * instantiated to physical pipes.  All it does is to keep track
 * of its ports so that methods like getInports() and outportIndex()
 * would return reasonable values.  This class is used in the
 * CompositeFactory class to represent the "inside interface" of a
 * composite pipe.
 * @author Chuan-kai Lin
 */
public class DummyLogicalPipe extends BaseLogicalPipe implements LogicalPipe {
	int portCount;
	
	public DummyLogicalPipe() {
		portCount = 0;
	}
	
	/**
	 * This method is specific to DummyLogicalPipe and is not a part
	 * of the LogicalPipe interface.  It allows the programmer to add
	 * a new inport to the set returned by getInports() and to make
	 * inportIndex() assign a port index to the new inport. 
	 * @param ident is the name of the new inport
	 */
	public void newInport(String ident) {
		if (inports.containsKey(ident))
			throw new RuntimeException();
		inports.put(ident, portCount++);
	}

	/**
	 * This method is specific to DummyLogicalPipe and is not a part
	 * of the LogicalPipe interface.  It allows the programmer to add
	 * a new outport to the set returned by getOutports() and to make
	 * outportIndex() assign a port index to the new outport. 
	 * @param ident is the name of the new outport
	 */
	public void newOutport(String ident) {
		if (outports.containsKey(ident))
			throw new RuntimeException();
		outports.put(ident, portCount++); 
	}
	
	/**
	 * A dummy pipe can assume any of the 2^n polarity configurations,
	 * which I represent as a PolaritySpace instance (which is much
	 * simpler and faster than inserting each of 2^n configurations
	 * into a PolaritySet).
	 */
	@Override
	public PolarityChoice validConfigurations() {
		return new PolaritySpace(portCount);
	}
	
	/**
	 * There is no way to instantiate a DummyLogicPipe. 
	 */
	@Override
	public PhysicalPipe instantiate(PolarityConfig config) {
		return null;
	}
}
