package infopipes.tests;

import infopipes.examples.PullSinkLogicalPipe;
import infopipes.examples.PullSinkPhysicalPipeP;
import infopipes.examples.PullSourceLogicalPipe;
import infopipes.examples.PushSinkLogicalPipe;
import infopipes.examples.PushSinkPhysicalPipeN;
import infopipes.examples.PushSourceLogicalPipe;
import infopipes.examples.PushSourcePhysicalPipeP;
import infopipes.pipes.CompositeFactory;
import infopipes.pipes.CompositeLogicalPipe;
import infopipes.pipes.CompositePhysicalPipe;
import infopipes.utils.Channel;
import infopipes.utils.PipePort;
import infopipes.utils.Polarity;
import infopipes.utils.PolarityConfig;
import junit.framework.TestCase;

public class CompositeTest extends TestCase {
	final PolarityConfig empty;
	
	public CompositeTest() {
		empty = new PolarityConfig(new Polarity[0]);
	}
	
	public void testSimplePushConnect() {
		CompositeFactory composite = new CompositeFactory();
		Channel ch = new Channel(new PipePort("src", "out"),
				                 new PipePort("sink", "in"));
		composite.addPipe("src", new PushSourceLogicalPipe());
		composite.addPipe("sink", new PushSinkLogicalPipe());
		composite.connect(ch);

		CompositeLogicalPipe logical = composite.instantiate();
		CompositePhysicalPipe pipe = logical.instantiate(empty);
		pipe.connected();
		PushSourcePhysicalPipeP src = (PushSourcePhysicalPipeP) pipe.getInternal("src");
		PushSinkPhysicalPipeN sink = (PushSinkPhysicalPipeN) pipe.getInternal("sink");
		src.doPush("message 1");
		src.doPush("message 2");
		assertTrue(sink.store.elementAt(0) == "message 1");
		assertTrue(sink.store.elementAt(1) == "message 2");
	}

	public void testSimplePullConnect() {
		CompositeFactory composite = new CompositeFactory();
		Channel ch = new Channel(new PipePort("src", "out"),
				                 new PipePort("sink", "in"));
		composite.addPipe("src", new PullSourceLogicalPipe());
		composite.addPipe("sink", new PullSinkLogicalPipe());
		composite.connect(ch);

		CompositeLogicalPipe logical = composite.instantiate();
		CompositePhysicalPipe pipe = logical.instantiate(empty);
		pipe.connected();
		PullSinkPhysicalPipeP sink = (PullSinkPhysicalPipeP) pipe.getInternal("sink");
		assertTrue(sink.doPull().equals("Packet 0"));
		assertTrue(sink.doPull().equals("Packet 1"));
	}
}
