package infopipes.utils;

import java.util.Arrays;

/**
 * This class represents a polarity configuration vector space in which
 * the polarity of each port is orthogonal to that of the other ports.
 * The PolaritySpace class is a substitute for PolaritySet with 2^n
 * powerset combination of unconstrained ports.   
 * @author Chuan-kai Lin
 */
public class PolaritySpace implements PolarityChoice {
	Polarity[] space;

	/**
	 * Create a completely unconstrained polarity space with ports
	 * numbered from 0 to size-1.
	 * @param size the number of ports in the space
	 */
	public PolaritySpace(int size) {
		space = new Polarity[size];
		Arrays.fill(space, Polarity.Unconstrained);
	}
	
	PolaritySpace(PolaritySpace input) {
		space = input.space.clone();
	}
	
	PolaritySpace() {}

	/**
	 * Constrain one dimension of the space to the given value.
	 */
	public void constrainPolarity(int index, Polarity value) {
		assert space[index].isUnconstrained();
		space[index] = value;
	}

	public Polarity testPolarity(int index) {
		return space[index];
	}

	public PolarityChoice copy() {
		return new PolaritySpace(this);
	}

	public Polarity[] one() {
		return space;
	}
}
